using System;
using System.Collections.Generic;
using System.Text.RegularExpressions;

namespace FaultTolerantWebService.Common
{
    /// <summary>
    /// Cette classe regroupe des mthodes statiques permettant de manipuler facilement des string
    /// 
    /// TODO : fusionner avec StringManager
    /// </summary>
    public static class TextUtil
    {
        /// <summary>
        /// Put the first char in upper case and the other chars in lowercase
        /// </summary>
        /// <param name="input"></param>
        /// <returns></returns>
        public static string ToUpperFirstChar(string input)
        {
            string result = input.ToLower();
            result = result[0].ToString().ToUpper() + Substring(result, 1, result.Length - 1);
            return result;
        }

        /// <summary>
        /// Met en majuscule la premire lettre de chaque mot
        /// </summary>
        /// <param name="wordList"></param>
        /// <returns></returns>
        public static string[] ToUpperFirstCharOfAllWords(string[] wordList)
        {
            List<string> result = new List<string>();
            foreach (string word in wordList)
            {
                result.Add(ToUpperFirstChar(word));
            }
            return result.ToArray();
        }

        /// <summary>
        /// Perform a substring but does not throw an exception if the length is greater than
        /// the string length
        /// </summary>
        /// <param name="input"></param>
        /// <param name="begin"></param>
        /// <param name="length"></param>
        /// <returns></returns>
        public static string Substring(string input, int begin, int length)
        {
            if (input == null) return null;
            if (input == "") return "";

            try
            {
                return input.Substring(begin, length);
            }
            catch (Exception)
            {
                try
                {
                    return input.Substring(begin);
                }
                catch (Exception)
                {
                    return "";
                }
            }
        }

        /// <summary>
        /// Retourne les length premiers caractres d'une chaine de caractre
        /// </summary>
        /// <param name="input"></param>
        /// <param name="length"></param>
        /// <returns></returns>
        public static string Left(string input, int length)
        {
            return Substring(input, 0, length);
        }

        /// <summary>
        /// Retourne les length derniers caractres d'une chaine de caractre
        /// </summary>
        /// <param name="input"></param>
        /// <param name="length"></param>
        /// <returns></returns>
        public static string Right(string input, int length)
        {
            return Substring(input, input.Length - length, length);
        }

        /// <summary>
        /// Retourne la premier lettre
        /// </summary>
        /// <param name="input"></param>
        /// <returns></returns>
        public static string First(string input)
        {
            return Left(input, 1);
        }

        /// <summary>
        /// Retourne la derniere lettre
        /// </summary>
        /// <param name="input"></param>
        /// <returns></returns>
        public static string Last(string input)
        {
            return Right(input, 1);
        }

        /// <summary>
        /// Return true if a string null or Empty
        /// </summary>
        /// <param name="text"></param>
        /// <returns></returns>
        public static bool IsEmpty(string text)
        {
            return text == null || text == string.Empty;
        }

        /// <summary>
        /// Return the input if it's not empty and the valueIfEmpty if it's empty
        /// </summary>
        /// <param name="input"></param>
        /// <param name="valueIfEmpty"></param>
        /// <returns></returns>
        public static string IfEmpty(string input, string valueIfEmpty)
        {
            return IsEmpty(input) ? valueIfEmpty : input;
        }
    }
}
