using System;
using System.Collections.Generic;
using System.Text;
using FaultTolerantWebService.Common;
using System.Reflection;
using FaultTolerantWebService.Generator;
using System.IO;

namespace FaultTolerantWebService.Console
{
    public class Interpreter
    {
        public Interpreter()
        {
        }

        /// <summary>
        /// Execute une gnration
        /// 
        /// 
        /// </summary>
        /// <param name="args"></param>
        public void Execute(string[] rawArgs)
        {
            GenerationArguments arguments = new GenerationArguments(rawArgs);

            Engine engine = new Engine(arguments.Language);

            if (arguments.Type.Count == 0)
                arguments.Type.AddRange(engine.GetSoapProxyList(arguments.Assembly));

            if (string.IsNullOrEmpty(arguments.OutputFile))
                arguments.OutputFile = "_FaultTolerantProxies.cs";

            using (Stream rawOutput = File.OpenWrite(arguments.OutputFile))
            {
                TextWriter output = new StreamWriter(rawOutput);

                foreach (Type t in arguments.Type)
                    engine.GenerateProxy(t, output);

                output.Flush();
            }
        }
    }

    /// <summary>
    /// Gestion des arguments de la lisgne de commande
    /// </summary>
    public class GenerationArguments
    {
        public readonly string langKey = "language";
        public readonly string assemblyKey = "assembly";
        public readonly string typesKey = "type";
        public readonly string outputKey = "output";


        private string language;
        private Assembly assembly;
        private List<Type> type;
        private string outputFile;

        public GenerationArguments(string[] rawArgs)
        {
            ParseRawArgs(rawArgs);
        }

        private void ParseRawArgs(string[] rawArgs)
        {
            Arguments args = new Arguments(rawArgs);
            if (args.Contains(langKey))
            {
                Language = args[langKey];
            }

            if (args.Contains(assemblyKey))
            {
                Assembly = System.Reflection.Assembly.LoadFrom(args[assemblyKey]);
            }

            if (args.Contains(typesKey))
            {
                Type.Clear();
                string[] typeNames = args[typesKey].Split(new char[] { ',' });
                foreach (string typeName in typeNames)
                    Type.Add(System.Type.GetType(typeName.Trim()));
            }

            if (args.Contains(outputKey))
            {
                OutputFile = args[outputKey];
            }
        }

        public Assembly Assembly
        {
            get { return this.assembly; }
            set { this.assembly = value; }
        }

        public string Language
        {
            get
            {
                if (string.IsNullOrEmpty(language)) language = "cs";
                return this.language;
            }
            set { this.language = value; }
        }

        public string OutputFile
        {
            get { return this.outputFile; }
            set { this.outputFile = value; }
        }

        public System.Collections.Generic.List<System.Type> Type
        {
            get
            {
                if (type == null) type = new List<Type>();
                return this.type;
            }
            set { this.type = value; }
        }

    }
}
