using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Web.Services.Protocols;
using System.Reflection;
using System.CodeDom;
using System.CodeDom.Compiler;
using System.IO;
using FaultTolerantWebService.Generator;
using System.Security.Policy;
using System.Diagnostics;
using FaultTolerantWebService.Common;

namespace FaultTolerantWebService.Ui
{
    /// <summary>
    /// todo :
    /// - rajouter le invoker dans la classe
    /// - faire ref  cet invoker au lieu de this
    /// - trouver la solution pour le new + virtual sur les mthodes
    /// </summary>
    public partial class MainForm : Form
    {
        public MainForm()
        {
            InitializeComponent();
        }

        protected override void OnLoad(EventArgs e)
        {
            base.OnLoad(e);

            Show();

            //enable state
            CheckEnableStates();

            //versions
            titleCtrl.Text += GetType().Assembly.GetName().Version;
            Text += GetType().Assembly.GetName().Version;

            //misc
            languageCtrl.SelectedIndex = 0;
            footerCtrl.Text = "Ready.";
        }

        #region events handling

        private void loadCtrl_Click(object sender, EventArgs e)
        {
            try
            {
                Generate(PromptAssemblyFileName(), languageCtrl.Text);
            }
            catch (CanceledException ex)
            {
                if (!ex.ByUser) MessageBox.Show("Action canceled : " + ex.Message);
            }
        }

        private void MainForm_Activated(object sender, EventArgs e)
        {
            CheckEnableStates();
            ReadClipboard();
        }

        private void refreshBtn_Click(object sender, EventArgs e)
        {
            Generate(assemblyLocationCtrl.Text, languageCtrl.Text);
        }

        private void quitCtrl_Click(object sender, EventArgs e)
        {
            Close();
        }

        private void saveAsCtrl_Click(object sender, EventArgs e)
        {
            SaveFileDialog dialog = new SaveFileDialog();
            dialog.Title = "Sauvegarder le resultat dans un fichier";
            dialog.InitialDirectory = Environment.GetFolderPath(Environment.SpecialFolder.Desktop);
            dialog.FileName = "FaultTolerantWebServices." + languageCtrl.Text;
            dialog.DefaultExt = "." + languageCtrl.Text;
            if (DialogResult.OK != dialog.ShowDialog()) return;

            File.WriteAllText(dialog.FileName, resultCtrl.Text);
        }

        private void copyCtrl_Click(object sender, EventArgs e)
        {
            Clipboard.SetText(resultCtrl.Text);
        }

        private void copyCmdCtrl_Click(object sender, EventArgs e)
        {
            Clipboard.SetText(commandLineCtrl.Text);
        }

        private void saveAsCmdCtrl_Click(object sender, EventArgs e)
        {
            SaveFileDialog dialog = new SaveFileDialog();
            dialog.Title = "Save command line into a batch file";
            dialog.InitialDirectory = Environment.GetFolderPath(Environment.SpecialFolder.Desktop);
            dialog.FileName = "GenerateFaultTolerantWebServices.bat";
            dialog.DefaultExt = ".bat";
            if (DialogResult.OK != dialog.ShowDialog()) return;

            File.WriteAllText(dialog.FileName, commandLineCtrl.Text);
        }

        private void loadFromClipboardCtrl_Click(object sender, EventArgs e)
        {
            ReadClipboard(true);
        }

        private void quitMnu_Click(object sender, EventArgs e)
        {
            Close();
        }

        private void refreshMnu_Click(object sender, EventArgs e)
        {
            Generate(assemblyLocationCtrl.Text, languageCtrl.Text);
        }

        private void aboutMnu_Click(object sender, EventArgs e)
        {
            Process.Start(new ProcessStartInfo(Constant.ProjectHomeUrl));
        }

        #endregion

        #region controller methods

        private void CheckEnableStates()
        {
            loadFromClipboardCtrl.Enabled = Clipboard.ContainsFileDropList();
            refreshBtn.Enabled = refreshMnu.Enabled = assemblyLocationCtrl.Text != "";
            copyCmdCtrl.Enabled = saveAsCmdCtrl.Enabled = commandLineCtrl.Text != "";
            copyCtrl.Enabled = saveAsCtrl.Enabled = resultCtrl.Text != "";
        }

        private void Generate(string assemblyFileName, string language)
        {
            try
            {
                if (!string.IsNullOrEmpty(assemblyFileName))
                {
                    //UI
                    assemblyLocationCtrl.Text = assemblyFileName;
                    commandLineCtrl.Text = GenerateCommandLine(assemblyFileName, language);
                    Update();
                    Application.DoEvents();

                    using (TempAppDomain temp = new TempAppDomain(GetWorkerDomain(assemblyFileName)))
                    {
                        Worker worker = (Worker)temp.Domain.CreateInstanceFromAndUnwrap(typeof(Worker).Assembly.Location, typeof(Worker).FullName);
                        resultCtrl.Text = worker.DoWork(assemblyFileName, language);
                    }

                    CheckEnableStates();
                }
            }
            catch (Exception ex)
            {
                MessageBox.Show("An error occured : " + ex.Message, "Error during generation.");
            }
        }

        private AppDomain GetWorkerDomain(string assemblyFileName)
        {
            AppDomain d = null;
            try
            {
                AppDomainSetup setup = new AppDomainSetup();
                setup.ApplicationBase = Path.GetDirectoryName(assemblyFileName);
                setup.PrivateBinPath = AppDomain.CurrentDomain.BaseDirectory;
                setup.ApplicationName = Path.GetFileNameWithoutExtension(assemblyFileName);

                d = AppDomain.CreateDomain(
                    "tempDomain",
                    null,
                    setup
                );

                d.Load(typeof(Worker).Assembly.GetName());
            }
            catch (Exception ex)
            {
                if (d != null) AppDomain.Unload(d);
                throw (ex);
            }
            return d;
        }

        private string PromptAssemblyFileName()
        {
            OpenFileDialog dialog = new OpenFileDialog();
            dialog.Title = "Selectionner le fichier contenant un assembly";
            dialog.InitialDirectory = Environment.GetFolderPath(Environment.SpecialFolder.Desktop);
            if (dialog.ShowDialog() != DialogResult.OK) throw new CanceledException("Annul par l'utilisateur", true);
            return dialog.FileName;
        }

        private void ReadClipboard()
        {
            ReadClipboard(false);
        }

        private void ReadClipboard(bool force)
        {
            if (Clipboard.ContainsFileDropList() && File.Exists(Clipboard.GetFileDropList()[0]))
            {
                if (assemblyLocationCtrl.Text == "" || force)
                {
                    assemblyLocationCtrl.Text = Clipboard.GetFileDropList()[0];
                    Generate(assemblyLocationCtrl.Text, languageCtrl.Text);
                }
            }
        }

        private string GenerateCommandLine(string assemblyFileName, string language)
        {
            //trying to be smart : bin/debug & bin/release are the std filespaths with visual studio
            //we want de proxy to be in the project root, not in the bin folder
            DirectoryInfo output = new DirectoryInfo(Path.GetDirectoryName(assemblyFileName));
            if (output.Name.ToLower() == "debug") output = output.Parent;
            if (output.Name.ToLower() == "release") output = output.Parent;
            if (output.Name.ToLower() == "bin") output = output.Parent;

            return
                Path.Combine(Application.StartupPath, "FaultTolerantWebService.Console.exe")
                + " --language=\"" + language + "\""
                + " --assembly=\"" + assemblyFileName + "\""
                + " --output=\"" + Path.Combine(output.FullName, "FaultTolerantProxies." + language) + "\"";
        }

        #endregion
    }
}